--Ursus c360
--#Autor: wisniagames239
--#data: 5/01/2021r.

C360 = {};

function C360.prerequisitesPresent(specializations)
	return true;
end;

function C360.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", C360);
	SpecializationUtil.registerEventListener(vehicleType, "onDelete", C360);
	SpecializationUtil.registerEventListener(vehicleType, "onReadStream", C360);
	SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", C360);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", C360);
end

function C360.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "setServiceAnimationState", C360.setServiceAnimationState);
end;


function C360:onLoad(savegame)
	self.spec_C360 = {};

	local spec = self.spec_C360;
	local specKey = "vehicle.C360";

	local triggers = {};
	triggers.playerTrigger = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, specKey .. ".service#playerTriggerNode"), self.i3dMappings);
end;

function C360:onDelete()
	local spec = self.spec_C360;

	if spec.serviceActivatable ~= nil then
		spec.serviceActivatable:delete();
	end;

	if spec.fuel ~= nil then
		if spec.fuelActivatable ~= nil then
			spec.fuelActivatable:delete();
		end;

		if self.isClient then
			g_soundManager:deleteSamples(spec.fuel.samples);
			ParticleUtil.deleteParticleSystem(spec.fuel.exhaustParticles);
			g_animationManager:deleteAnimations(spec.fuel.animationNodes);
		end;
	end;
end;

function C360:saveToXMLFile(xmlFile, key, usedModNames)
	local spec = self.spec_C360;

	if spec.serviceAnimation ~= nil and spec.serviceAnimation.markers ~= nil then
		setXMLBool(xmlFile, key .. ".triggerMarkers#serviceActive", spec.serviceAnimation.markers.showMarkers);
	end;

	if spec.fuel ~= nil and spec.fuel.markers ~= nil then
		setXMLBool(xmlFile, key .. ".triggerMarkers#fuelActive", spec.fuel.markers.showMarkers);
	end;
end;

function C360:onReadStream(streamId, connection)
	local spec = self.spec_C360;

	if spec.serviceAnimation ~= nil then
		local isActive = streamReadBool(streamId);
		self:setServiceAnimationState(isActive, true);

		if spec.serviceAnimation.markers ~= nil then
			local showMarkers = streamReadBool(streamId);
			self:setServiceMarkerState(showMarkers, true, true);
		end;
	end;

	if spec.fuel ~= nil then
		local isActive = streamReadBool(streamId);
		self:setServiceFuelState(isActive, true);

		if spec.fuel.markers ~= nil then
			local showMarkers = streamReadBool(streamId);
			self:setServiceMarkerState(showMarkers, false, true);
		end;
	end;
end;

function C360:onWriteStream(streamId, connection)
	local spec = self.spec_C360;

	if spec.serviceAnimation ~= nil then
		local isActive = spec.serviceAnimation.isActive;
		streamWriteBool(streamId, isActive);

		if spec.serviceAnimation.markers ~= nil then
			local markersActive = spec.serviceAnimation.markers.showMarkers;
			streamWriteBool(streamId, markersActive);
		end;
	end;

	if spec.fuel ~= nil then
		local isActive = spec.fuel.isActive;
		streamWriteBool(streamId, spec.fuel.isActive);

		if spec.fuel.markers ~= nil then
			local markersActive = spec.fuel.markers.showMarkers;
			streamWriteBool(streamId, markersActive);
		end;
	end;
end;

function C360:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if self.isClient then
		local raiseActive = false;
		local spec = self.spec_C360;

	
		if spec.serviceActivatable ~= nil then
			if spec.serviceAnimation ~= nil and spec.serviceAnimation.markers ~= nil then
				local serviceMarkers = spec.serviceAnimation.markers;
				local animationTime = self:getAnimationTime(spec.serviceAnimation.name);
				spec.serviceAnimation.finished = animationTime == 1;

				local vehiclesInTrigger = spec.serviceActivatable.currentVehicle ~= nil;
				if spec.serviceAnimation.finished then
					raiseActive = self:setServiceMarker(serviceMarkers, vehiclesInTrigger, spec.serviceAnimation.isActive, false, raiseActive);
				else
					raiseActive = self:setServiceMarker(serviceMarkers, vehiclesInTrigger, false, spec.serviceAnimation.isActive, raiseActive);
				end;
			end;

			if spec.serviceActivatable.currentVehicle ~= nil and spec.serviceInformation ~= nil then
				if spec.serviceInformation.headers ~= nil then
					local numHeaders = #spec.serviceInformation.headers
					if numHeaders > 0 then
						for i = 1, numHeaders do
							C360:drawServiceInformation(self, spec.serviceInformation.headers[i]);
						end;

						raiseActive = true;
					end;
				end;

				if spec.serviceInformation.texts ~= nil then
					local numTexts = #spec.serviceInformation.texts
					local currentVehicle = spec.serviceActivatable.currentVehicle;
					local storeItem = g_storeManager:getItemByXMLFilename(currentVehicle.configFileName:lower());
					if numTexts > 0 then
						for i = 1, numTexts do
							local serviceInfo = spec.serviceInformation.texts[i];
							if serviceInfo.keyName == "vehicleOwner" then
								local farm = g_farmManager:getFarmById(currentVehicle:getActiveFarm());
								-- g_currentMission.player.visualInformation.playerName
								serviceInfo.text = Utils.getNoNil(farm.name, "N/A");
							elseif serviceInfo.keyName == "vehicleName" then
								serviceInfo.text = string.format("%s  %s", g_brandManager:getBrandByIndex(storeItem.brandIndex).name, storeItem.name);
							elseif serviceInfo.keyName == "vehicleOperatingTime" then
								-- This is the same look as the vehicle hud. Store gui is wrong using 00:00 h
								local minutes = currentVehicle.operatingTime / (1000 * 60);
								local hours = math.floor(minutes / 60);
								minutes = math.floor((minutes - hours * 60) / 6);
								serviceInfo.text = string.format("%02d.%01d h", hours, minutes);
							elseif serviceInfo.keyName == "vehicleFuel" then
								if currentVehicle.spec_motorized ~= nil and currentVehicle.spec_motorized.consumersByFillTypeName ~= nil then
									local fillUnitIndex = currentVehicle.spec_motorized.consumersByFillTypeName.diesel.fillUnitIndex;
									local fillLevel = currentVehicle:getFillUnitFillLevel(fillUnitIndex);
									serviceInfo.text = g_i18n:formatVolume(fillLevel, 2);
								else
									serviceInfo.text = "N/A";
								end;
							elseif serviceInfo.keyName == "vehicleAge" then
								serviceInfo.text = string.format(g_i18n:getText("shop_age"), currentVehicle.age);
							elseif serviceInfo.keyName == "vehicleCondition" then
								local damagedPercent = math.floor(math.min(currentVehicle:getWearTotalAmount() * 100, 100));
								local conditionPercent = tostring(100 - damagedPercent);
								serviceInfo.text = conditionPercent .. " %";
							end;

							C360:drawServiceInformation(self, serviceInfo);
						end;

						raiseActive = true;
					end;
				end;
			end;
		end;

		if raiseActive then
			self:raiseActive();
		end;
	end;
end;



C360AnimationEvent = {};
C360AnimationEvent_mt = Class(C360AnimationEvent, Event);
InitEventClass(C360AnimationEvent, "C360AnimationEvent");

function C360AnimationEvent:emptyNew()
	local self = Event:new(C360AnimationEvent_mt);
	return self;
end;

function C360AnimationEvent:new(object, isActive)
	local self = C360AnimationEvent:emptyNew()
	self.object = object;
	self.isActive = isActive;
	return self;
end;

function C360AnimationEvent:readStream(streamId, connection)
	self.object = NetworkUtil.readNodeObject(streamId);
	self.isActive = streamReadBool(streamId);
	self:run(connection);
end;

function C360AnimationEvent:writeStream(streamId, connection)
	NetworkUtil.writeNodeObject(streamId, self.object);
	streamWriteBool(streamId, self.isActive);
end;

function C360AnimationEvent:run(connection)
	self.object:setServiceAnimationState(self.isActive, true);

	if not connection:getIsServer() then
		g_server:broadcastEvent(C360AnimationEvent:new(self.object, self.isActive), nil, connection, self.object);
	end;
end;

function C360AnimationEvent.sendEvent(vehicle, isActive, noEventSend)
	if vehicle.spec_C360 ~= nil and vehicle.spec_C360.serviceAnimation ~= nil and
		vehicle.spec_C360.serviceAnimation.isActive ~= isActive then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(C360AnimationEvent:new(vehicle, isActive), nil, nil, vehicle);
			else
				g_client:getServerConnection():sendEvent(C360AnimationEvent:new(vehicle, isActive));
			end;
		end;
	end;
end;



